@@ -36,6 +36,7 @@
 typedef struct
 {
   GtkBox *box;
+  GtkLabel *description;
   GtkWidget *scrolled_window;
 
   char *icon_name;
@@ -59,6 +60,7 @@ enum {
   PROP_0,
   PROP_ICON_NAME,
   PROP_TITLE,
+  PROP_DESCRIPTION,
   PROP_NAME,
   PROP_USE_UNDERLINE,
   LAST_PROP,
@@ -66,6 +68,14 @@ enum {
 
 static GParamSpec *props[LAST_PROP];
 
+static gboolean
+is_visible_group (GtkWidget *widget,
+                  gpointer   user_data)
+{
+  return ADW_IS_PREFERENCES_GROUP (widget) &&
+         gtk_widget_get_visible (widget);
+}
+
 static void
 adw_preferences_page_get_property (GObject    *object,
                                    guint       prop_id,
@@ -81,6 +91,9 @@ adw_preferences_page_get_property (GObject    *object,
   case PROP_TITLE:
     g_value_set_string (value, adw_preferences_page_get_title (self));
     break;
+  case PROP_DESCRIPTION:
+    g_value_set_string (value, adw_preferences_page_get_description (self));
+    break;
   case PROP_NAME:
     g_value_set_string (value, adw_preferences_page_get_name (self));
     break;
@@ -107,6 +120,9 @@ adw_preferences_page_set_property (GObject      *object,
   case PROP_TITLE:
     adw_preferences_page_set_title (self, g_value_get_string (value));
     break;
+  case PROP_DESCRIPTION:
+    adw_preferences_page_set_description (self, g_value_get_string (value));
+    break;
   case PROP_NAME:
     adw_preferences_page_set_name (self, g_value_get_string (value));
     break;
@@ -151,6 +167,7 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
   object_class->finalize = adw_preferences_page_finalize;
 
   widget_class->compute_expand = adw_widget_compute_expand;
+  widget_class->focus = adw_widget_focus_child;
 
   /**
    * AdwPreferencesPage:icon-name: (attributes org.gtk.Property.get=adw_preferences_page_get_icon_name org.gtk.Property.set=adw_preferences_page_set_icon_name)
@@ -172,6 +189,18 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
                          "",
                          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  /**
+   * AdwPreferencesPage:description: (attributes org.gtk.Property.get=adw_preferences_page_get_description org.gtk.Property.set=adw_preferences_page_set_description)
+   *
+   * The description to be displayed at the top of the page.
+   * 
+   * Since: 1.4
+   */
+  props[PROP_DESCRIPTION] =
+    g_param_spec_string ("description", NULL, NULL,
+                         "",
+                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+
   /**
    * AdwPreferencesPage:name: (attributes org.gtk.Property.get=adw_preferences_page_get_name org.gtk.Property.set=adw_preferences_page_set_name)
    *
@@ -197,6 +226,7 @@ adw_preferences_page_class_init (AdwPreferencesPageClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class,
                                                "/org/gnome/Adwaita/ui/adw-preferences-page.ui");
   gtk_widget_class_bind_template_child_private (widget_class, AdwPreferencesPage, box);
+  gtk_widget_class_bind_template_child_private (widget_class, AdwPreferencesPage, description);
   gtk_widget_class_bind_template_child_private (widget_class, AdwPreferencesPage, scrolled_window);
 
   gtk_widget_class_set_css_name (widget_class, "preferencespage");
@@ -208,6 +238,7 @@ static void
 adw_preferences_page_init (AdwPreferencesPage *self)
 {
   AdwPreferencesPagePrivate *priv = adw_preferences_page_get_instance_private (self);
+
   priv->title = g_strdup ("");
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -330,12 +361,9 @@ adw_preferences_page_set_icon_name (AdwPreferencesPage *self,
 
   priv = adw_preferences_page_get_instance_private (self);
 
-  if (g_strcmp0 (priv->icon_name, icon_name) == 0)
+  if (!g_set_str (&priv->icon_name, icon_name))
     return;
 
-  g_clear_pointer (&priv->icon_name, g_free);
-  priv->icon_name = g_strdup (icon_name);
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_ICON_NAME]);
 }
 
@@ -376,15 +404,65 @@ adw_preferences_page_set_title (AdwPreferencesPage *self,
 
   priv = adw_preferences_page_get_instance_private (self);
 
-  if (g_strcmp0 (priv->title, title) == 0)
+  if (!g_set_str (&priv->title, title ? title : ""))
     return;
 
-  g_clear_pointer (&priv->title, g_free);
-  priv->title = g_strdup (title ? title : "");
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_TITLE]);
 }
 
+/**
+ * adw_preferences_page_get_description: (attributes org.gtk.Method.get_property=description)
+ * @self: a preferences page
+ *
+ * Gets the description of @self.
+ *
+ * Returns: the description of @self.
+ * 
+ * Since: 1.4
+ */
+const char *
+adw_preferences_page_get_description (AdwPreferencesPage *self)
+{
+  AdwPreferencesPagePrivate *priv;
+
+  g_return_val_if_fail (ADW_IS_PREFERENCES_PAGE (self), NULL);
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  return gtk_label_get_text (priv->description);
+}
+
+/**
+ * adw_preferences_page_set_description: (attributes org.gtk.Method.set_property=description)
+ * @self: a preferences page
+ * @description: the description
+ *
+ * Sets the description of @self.
+ * 
+ * The description is displayed at the top of the page.
+ * 
+ * Since: 1.4
+ */
+void
+adw_preferences_page_set_description (AdwPreferencesPage *self,
+                                      const char         *description)
+{
+  AdwPreferencesPagePrivate *priv;
+
+  g_return_if_fail (ADW_IS_PREFERENCES_PAGE (self));
+
+  priv = adw_preferences_page_get_instance_private (self);
+
+  if (g_strcmp0 (gtk_label_get_label (priv->description), description) == 0)
+    return;
+
+  gtk_label_set_label (priv->description, description);
+  gtk_widget_set_visible (GTK_WIDGET (priv->description),
+                          description && *description);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_DESCRIPTION]);
+}
+
 /**
  * adw_preferences_page_get_name: (attributes org.gtk.Method.get_property=name)
  * @self: a preferences page
@@ -422,12 +500,9 @@ adw_preferences_page_set_name (AdwPreferencesPage *self,
 
   priv = adw_preferences_page_get_instance_private (self);
 
-  if (g_strcmp0(priv->name, name) == 0)
+  if (!g_set_str (&priv->name, name))
     return;
 
-  g_clear_pointer (&priv->name, g_free);
-  priv->name = g_strdup (name);
-
   g_object_notify_by_pspec (G_OBJECT (self), props[PROP_NAME]);
 }
 
@@ -501,16 +576,16 @@ adw_preferences_page_get_rows (AdwPreferencesPage *self)
 {
   AdwPreferencesPagePrivate *priv;
   GListModel *model;
-  GtkExpression *expr;
+  GtkCustomFilter *filter;
 
   g_return_val_if_fail (ADW_IS_PREFERENCES_PAGE (self), NULL);
 
   priv = adw_preferences_page_get_instance_private (self);
 
-  expr = gtk_property_expression_new (GTK_TYPE_WIDGET, NULL, "visible");
+  filter = gtk_custom_filter_new ((GtkCustomFilterFunc) is_visible_group, NULL, NULL);
 
   model = gtk_widget_observe_children (GTK_WIDGET (priv->box));
-  model = G_LIST_MODEL (gtk_filter_list_model_new (model, GTK_FILTER (gtk_bool_filter_new (expr))));
+  model = G_LIST_MODEL (gtk_filter_list_model_new (model, GTK_FILTER (filter)));
   model = G_LIST_MODEL (gtk_map_list_model_new (model,
                                                 (GtkMapListModelMapFunc) preferences_group_to_rows,
                                                 NULL,
